"""
Module contains constants
"""

INIT_REDIS_LABEL = 'INIT_REDIS'
INIT_MONGO_LABEL = 'INIT_MONGO'

EVENTS_USER_AGENT = 'user_agent'
EVENTS_DATE = 'date_event'
EVENTS_IP = 'user_ip'
EVENTS_URL = 'handler_url'
EVENTS_SRV_NAME = 'server_name'
EVENTS_METHOD = 'method'
